@@ -37,90 +37,115 @@
 #include <sys/socket.h>
 #include <sys/param.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
 #include "yp_misc.h"
 #include "yp_networks.h"
 #include "rpcsvc/ypclnt.h"
 
-
-static struct netent *netent_parse(char *str, int len)
-{
-    static struct netent net;
-    static char buf[2048], temp[2048], nname[YP_MAX_NET_NAME_LEN];
-    static char *alias_list[YP_MAX_NET_ALIASES];
-    static int i, j, shiftcount;
-    static unsigned long addr;
-            
-    /* Can't handle this -> let's just forget it... */
-    if (len > 2040)
-	return NULL;
-
-    strncpy(buf, str, len);
-
-    /*
-     * Copy out name
-     */
-    net.n_name = nname;
-    for (i = 0 ; (!(isspace(buf[i])) && (i < len)) ; ++i)
-    {
-       net.n_name[i] = buf[i];
-    }
-    net.n_name[i] = '\0';
-    
-    
-    /*
-     * Eat white space
-     */
-    for ( ; ((isspace(buf[i])) && (i < len)) ; ++i)
-       ;
-
-#ifdef LSB
-    /* 
-     * Copy and build net address
-     */
-    addr       = 0;
-    shiftcount = 24;
-    while (1)
-    {
-       for (j = 0; ((isdigit(buf[i])) && (j < 3)) ; ++i, ++j)
-	  temp[j] = buf[i];
-       temp[j] = '\0';
-       
-       addr |= (atoi(temp) << shiftcount);
-
-       if ((buf[i] != '.') || (shiftcount <= 0))
-	  break;
-       else
-	  ++i;
-
-       shiftcount -= 8;
-    }
-
-    net.n_net = htonl(addr);
-#else
-#error No MSB support, yet.
-#endif
-
-    net.n_addrtype = AF_INET;
-    net.n_aliases  = alias_list;
-
-    j = 0;
-    net.n_aliases[j] = _yp_strip_names(_yp_xstrtok(buf+i, ' '));
-    while ((net.n_aliases[j] = _yp_strip_names(_yp_xstrtok(NULL, ' '))))
-    {
-       if (strlen(net.n_aliases[j]) > 0)
-	   ++j;
-    }
-    net.n_aliases[j] = 0;
-
-    
-    return &net;
-    
-}
+#include "xalloc.h"
 
 
 static int rewind_flag = 1;
 static char *savekey = NULL;
 static int savekeylen = 0;
+static char *pbuf = NULL;
+
+#ifdef HAVE_BUGGY_INET_NETWORK
+static int _inet_network(char *cp)
+{
+    int v0, v1, v2, v3;
+    int res;
+    
+    v0 = v1 = v2 = v3 = 0;
+
+    if (sscanf(cp, "%d.%d.%d.%d", &v0, &v1, &v2, &v3) <= 0)
+	return -1;
+    
+    res = 0;
+    
+    if (v0|v1|v2|v3)
+    {
+	res = v0;
+
+	if (v1|v2|v3)
+	{
+	    res <<= 8;
+	    res += v1;
+
+	    if (v2|v3)
+	    {
+		res <<= 8;
+		res += v2;
+
+		if (v3)
+		{
+		    res <<= 8;
+		    res += v3;
+		}
+	    }
+	}
+    }
+    
+    return res;
+}
+
+#ifdef inet_network
+#undef inet_network
+#endif
+#define inet_network _inet_network
+
+#endif
+
+static struct netent *netent_parse(char *str, int len)
+{
+    static struct netent ne;
+    char *cp;
+    int alen;
+    int i;
+    
+    
+    if (Xalloc(&pbuf, len) == NULL)
+	return NULL;
+    
+    strncpy(pbuf, str, len);
+    pbuf[len] = '\0';
+    
+    ne.n_name = _yp_xstrtok(pbuf, ' ');
+    if (ne.n_name == NULL)
+	return NULL;
+
+    ne.n_addrtype = AF_INET;
+    
+    cp = _yp_xstrtok(NULL, ' ');
+    if (cp == NULL)
+	return NULL;
+
+    ne.n_net = inet_network(cp);
+
+    /* Build alias list */
+    alen = 1;
+    if (Xalloc(&ne.n_aliases, alen+1) == NULL)
+	return NULL;
+
+    i = 0;
+    while ((cp = _yp_xstrtok(NULL, ' ')) && cp[0] != '#')
+    {
+	if (i >= alen)
+	{
+	    alen += 10;
+	    if (Xalloc(&ne.n_aliases, alen+1) == NULL)
+		return NULL;
+	}
+	
+	ne.n_aliases[i++] = cp;
+    }
+
+
+    ne.n_aliases[i] = NULL;
+
+    return &ne;
+}
+
 
 
 void yp_setnetent(int stayopen, void *opts)
@@ -192,32 +217,44 @@ struct netent *yp_getnetent(void *opts)
 
 struct netent *yp_getnetbyaddr(long net, int type, void *opts)
 {
-    struct netent *netd;
+    struct netent *nep;
     char *map;
     char *domain;
     char *result;
     int len;
     char buf[256];
-
+    int blen;
+    struct in_addr in;
+    
 
     map = _ypopts_getmd(opts, "networks", ".byaddr", &domain);
     if (map == NULL)
 	return NULL;
 
-    sprintf(buf, "%d", net);
-
-    net = 0;
-        
+    in = inet_makeaddr(net, 0);
+    strcpy(buf, inet_ntoa(in));
+    blen = strlen(buf);
+    
+    nep = 0;
+    
+  Retry:
     if (yp_match(domain, map, buf, strlen(buf), &result, &len) == 0)
     {
-	netd = netent_parse(result, len);
+	nep = netent_parse(result, len);
 	free(result);
     }
+    else if (buf[blen-2] == '.' && buf[blen-1] == '0')
+    {
+	/* Try again, but with trailing dot(s) removed (one by one) */
+	buf[blen-2] = '\0';
+	blen -= 2;
+	goto Retry;
+    }
 
     free(map);
     free(domain);
     
-    return netd;
+    return nep;
 }
 
 
