@@ -32,42 +32,65 @@
 #include <errno.h>
 #include <string.h>
 #include <stdlib.h>
+#include <ctype.h>
 #include "yp_misc.h"
 #include "yp_passwd.h"
 #include "rpcsvc/ypclnt.h"
 
+#include "xalloc.h"
+
+static int rewind_flag = 1;
+static char *savekey = NULL;
+static int savekeylen = 0;
+static char *pbuf = NULL;
+
 
 static struct passwd *pwent_parse(char *str, int len)
 {
     static struct passwd pwd;
-    extern char *strtok();
-    static char buf[2048];
-    
+    char *cp;
 
-    /* Can't handle this -> let's just forget it... */
-    if (len > 2040)
+    
+    if (Xalloc(&pbuf, len) == NULL)
+	return NULL;
+    
+    strncpy(pbuf, str, len);
+    pbuf[len] = '\0';
+    
+    pwd.pw_name   = _yp_xstrtok(pbuf, ':');
+    if (pwd.pw_name == NULL)
 	return NULL;
-
-    strncpy(buf, str, len);
-    buf[len] = '\0';
     
-    pwd.pw_name   = _yp_xstrtok(buf, ':');
     pwd.pw_passwd = _yp_xstrtok(NULL, ':');
-    pwd.pw_uid    = atoi(_yp_xstrtok(NULL, ':'));
-    pwd.pw_gid    = atoi(_yp_xstrtok(NULL, ':'));
+    if (pwd.pw_passwd == NULL)
+	return NULL;
+    
+    cp = _yp_xstrtok(NULL, ':');
+    if (cp == NULL || !isdigit(*cp))
+	return NULL;
+    pwd.pw_uid    = atoi(cp);
+
+    cp = _yp_xstrtok(NULL, ':');
+    if (cp == NULL || !isdigit(*cp))
+	return NULL;
+    pwd.pw_gid    = atoi(cp);
+    
     pwd.pw_gecos  = _yp_xstrtok(NULL, ':');
+    if (pwd.pw_gecos == NULL)
+	return NULL;
+    
     pwd.pw_dir    = _yp_xstrtok(NULL, ':');
+    if (pwd.pw_dir == NULL)
+	return NULL;
+    
     pwd.pw_shell  = _yp_xstrtok(NULL, ':');
+    if (pwd.pw_dir == NULL)
+	return NULL;
 
     return &pwd;
 }
 
 
-static int rewind_flag = 1;
-static char *savekey = NULL;
-static int savekeylen = 0;
-
-
 void yp_setpwent(void *opts)
 {
     rewind_flag = 1;
@@ -124,7 +147,29 @@ struct passwd *yp_getpwent(void *opts)
 	savekeylen = keylen;
     }
 
-    pw = pwent_parse(result, len);
+    /*
+    ** Loop, fetching the next entry if there is an incorrectly
+    ** formatted entry.
+    */
+    errno = 0;
+    while ((pw = pwent_parse(result, len)) == NULL && errno == 0)
+    {
+#ifdef DEBUG
+	fprintf(stderr, "yp_passwd: Invalid passwd entry: %.*s\n",
+		len, result);
+#endif
+	free(result);
+	
+	if (yp_next(domain, map,
+		    savekey, savekeylen, &outkey, &keylen,
+		    &result, &len))
+	    goto error;
+	
+	free(savekey);
+	savekey = outkey;
+	savekeylen = keylen;
+    }
+    
     free(result);
 
   error:
@@ -142,7 +187,7 @@ struct passwd *yp_getpwuid(uid_t uid, void *opts)
     char *domain;
     char *result;
     int len;
-    char buf[256];
+    char buf[16];
 
 
     map = _ypopts_getmd(opts, "passwd", ".byuid", &domain);
