@@ -26,8 +26,9 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <linux/socket.h>
-#include <linux/in.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
 #include "yp_hosts.h"
 #include "yp_misc.h"
 
@@ -41,7 +42,7 @@ main(int argc, char *argv[])
    unsigned long test;
 
    if (argc >= 2)
-      test = htonl(_yp_readaddr(argv[1]));
+      test = inet_addr(argv[1]);
 
    host = yp_gethostbyaddr((char *) &test, sizeof(unsigned long),
 			   AF_INET, NULL);
