@@ -24,31 +24,88 @@
 **          Michael A. Griffith <grif@cs.ucr.edu>  
 */
 
+#include "config.h"
+
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
 #include "yp_networks.h"
 #include "yp_misc.h"
 
 extern void print_network(struct netent *);
 
+#ifdef HAVE_BUGGY_INET_NETWORK
+static int _inet_network(char *cp)
+{
+    int v0, v1, v2, v3;
+    int res = 0;
+    
+    v0 = v1 = v2 = v3 = 0;
+
+    if (sscanf(cp, "%d.%d.%d.%d", &v0, &v1, &v2, &v3) <= 0)
+	return -1;
+
+    res = 0;
+    if (v0|v1|v2|v3)
+    {
+	res = v0;
+
+	if (v1|v2|v3)
+	{
+	    res <<= 8;
+	    res += v1;
+
+	    if (v2|v3)
+	    {
+		res <<= 8;
+		res += v2;
+
+		if (v3)
+		{
+		    res <<= 8;
+		    res += v3;
+		}
+	    }
+	}
+    }
+    
+    return res;
+}
+
+#ifdef inet_network
+#undef inet_network
+#endif
+#define inet_network _inet_network
+
+#endif
+
 
 int
 main(int argc, char *argv[])
 {
    struct netent *net;
    unsigned long test;
-      
-   if (argc >= 2)
-      test = htonl(_yp_readaddr(argv[1]));
-   
-    net = yp_getnetbyaddr(test, AF_INET, NULL);
+   struct in_addr in;
 
-    if (net)
-	print_network(net);
-    else
-	printf("No entry found\n");
-    
-    exit(0);
+   
+   if (argc < 2)
+       exit(1);
+   
+   test = inet_network(argv[1]);
+   printf("%04x\n", test);
+   
+   in = inet_makeaddr(test, 0);
+   puts(inet_ntoa(in));
+   
+   net = yp_getnetbyaddr(test, AF_INET, NULL);
+
+   if (net)
+       print_network(net);
+   else
+       printf("No entry found\n");
+   
+   exit(0);
 }
